package tofly.zggas.zggaspay.controller;

import com.google.zxing.BarcodeFormat;
import com.google.zxing.EncodeHintType;
import com.google.zxing.MultiFormatWriter;
import com.google.zxing.WriterException;
import com.google.zxing.client.j2se.MatrixToImageWriter;
import com.google.zxing.common.BitMatrix;
import com.google.zxing.qrcode.decoder.ErrorCorrectionLevel;
import com.wechat.pay.java.core.exception.ServiceException;
import com.wechat.pay.java.service.payments.nativepay.model.PrepayResponse;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import tofly.zggas.zggaspay.service.PayService;
import tofly.zggas.zggaspay.service.entity.dto.TradeBill;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.OutputStream;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

@Slf4j
@RestController
@RequestMapping("/v1/pay")
@AllArgsConstructor
public class PayController {

    private final PayService payService;


    @GetMapping("/test")
    public void test() {
        payService.test();
    }

    @GetMapping("/prepay")
    @PostMapping("/prepay")
    public PrepayResponse prepay(@RequestParam String woId,
                                 @RequestParam(required = false) Integer total,
                                 @RequestParam(required = false) String attach) {
        return payService.prepay(woId, total, attach);
    }

    @GetMapping("/downloadCertificate")
    @PostMapping("/downloadCertificate")
    public void downloadCertificate() {
        payService.downloadCertificate();
    }

    @GetMapping("/prepayCode")
    @PostMapping("/prepayCode")
    public void prepayCode(@RequestParam String woId,
                           @RequestParam(required = false) Integer total,
                           @RequestParam(required = false) String attach,
                           HttpServletResponse response) throws IOException, WriterException {
        toQRcode(payService.prepay(woId, total, attach).getCodeUrl(), response.getOutputStream());
    }

    @GetMapping("/checkAccount")
    @PostMapping("/checkAccount")
    public ResponseEntity checkAccount(@RequestParam String billDate) {
        Map<String, Object> result = new LinkedHashMap<>();
        try {
            payService.checkAccount(billDate);
            result.put("code", "SUCCESS");
            result.put("message", "对账处理成功");
            log.info("对账处理成功");
            return new ResponseEntity<Map>(result, HttpStatus.OK);
        } catch (ServiceException e) {
            log.error("对账处理失败", e);
            result.put("code", "FAIL");
            result.put("message", String.format("对账处理失败:%s", e.getMessage()));
            return new ResponseEntity<Map>(result, HttpStatus.OK);
        } catch (Exception e) {
            log.error("对账处理失败", e);
            result.put("code", "FAIL");
            result.put("message", String.format("对账处理失败:%s", e.getMessage()));
            return new ResponseEntity<Map>(result, HttpStatus.INTERNAL_SERVER_ERROR);
        }
    }

    @GetMapping("/getTradeBill")
    @PostMapping("/getTradeBill")
    public List<TradeBill> getTradeBill(@RequestParam String billDate) {
        return payService.getTradeBill(billDate);
    }

    /**
     * 关闭订单
     */
    @PostMapping("/closeOrder")
    public void closeOrder(@RequestParam String outTradeNo) {
        payService.closeOrder(outTradeNo);
    }

    /**
     * 通知地址
     */
    @PostMapping("/notify")
    public ResponseEntity notify2(
            @RequestHeader("Wechatpay-Signature") String signature,
            @RequestHeader("Wechatpay-Nonce") String nonce,
            @RequestHeader("Wechatpay-Timestamp") String timestamp,
            @RequestHeader("Wechatpay-Signature-Type") String signType,
            @RequestHeader("Wechatpay-Serial") String serial,
            @RequestBody String requestBody) {
        Map<String, Object> result = new LinkedHashMap<>();
        try {
            String outTradeNo = payService.notify_(signType, nonce, timestamp, requestBody, signature, serial);
            result.put("code", "SUCCESS");
            log.error("支付通知处理成功:{}", outTradeNo);
            return new ResponseEntity<Map>(result, HttpStatus.OK);
        } catch (Exception e) {
            log.error("支付通知处理失败", e);
            result.put("code", "FAIL");
            result.put("message", e.getMessage());
            return new ResponseEntity<Map>(result, HttpStatus.INTERNAL_SERVER_ERROR);
        }

    }

    private void toQRcode(String url, OutputStream outputStream) throws WriterException, IOException {
        int width = 200; // 图像宽度
        int height = 200; // 图像高度
        String format = "gif";// 图像类型
        Map<EncodeHintType, Object> hints = new HashMap<>();
        //内容编码格式
        hints.put(EncodeHintType.CHARACTER_SET, "UTF-8");
        // 指定纠错等级
        hints.put(EncodeHintType.ERROR_CORRECTION, ErrorCorrectionLevel.H);
        //设置二维码边的空度，非负数
        hints.put(EncodeHintType.MARGIN, 1);
        BitMatrix bitMatrix = new MultiFormatWriter().encode(url, BarcodeFormat.QR_CODE, width, height, hints);
        MatrixToImageWriter.writeToStream(bitMatrix, format, outputStream);// 输出原图片
    }

}
