package tofly.zggas.zggaspay.service;

import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;

import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.time.format.DateTimeFormatter;
import java.util.concurrent.TimeUnit;

/***
 * 支付异步任务
 */
@Slf4j
@Service
@AllArgsConstructor
public class PayTask {

    private final PayService payService;

    @Scheduled(fixedRate = 10, timeUnit = TimeUnit.SECONDS)
    public void check() {
        log.info("check...");
        payService.check();
    }

    @Scheduled(fixedRate = 60, timeUnit = TimeUnit.SECONDS)
    public void downloadCertificate() {
        log.info("downloadCertificate...");
        payService.downloadCertificate();
    }

    //微信在次日9点启动生成前一天的对账单
    @Scheduled(cron = "0 0/30 10,11 * * ?")
    public void checkAccount() {
        log.info("checkAccount...");
        LocalDate now = LocalDate.now();
        String billDate = now.plusDays(-1).format(DateTimeFormatter.ofPattern("yyyy-MM-dd"));
        try {
            payService.checkAccount(billDate);
            log.info("对账处理成功");
        } catch (Throwable t) {
            log.error("对账处理异常", t);
        }
    }

}
